/******************************************************************************
* bypass_intersections.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * bypass_intersections.h
 *
 *  Created on: Jul 1, 2010
 *      Author: pajor
 */

#ifndef BYPASS_INTERSECTIONS_H_
#define BYPASS_INTERSECTIONS_H_

#include "../dataStructures/path.h"
#include "../geometry/tools.h"

#include <vector>

namespace schematization {

// This algorithm removes self-intersections by bypassing them.
// Only self-intersections are bypassed that are below a certain threshold parameter
// Returns the number of bypassed intersections
template<typename pathType>
class bypass_intersections {

public:

public:
	bypass_intersections(pathType &p) : input_path(p) {
	}


	// The main routine to bypass intersections
	size_t run(const double max_len) {
		simplified_path = input_path;
		pathType &path = simplified_path; // only an alias

		size_t removed_intersections(0);
		for (size_t n = 1; n < path.size(); ++n) {
			for (size_t m = n+2; m < path.size(); ++m) {

				// Define lines induced by the currently considered segments on the path.
				line l1(path[n-1].coord, path[n].coord);
				line l2(path[m-1].coord, path[m].coord);

				// Test for an intersection between l1 and l2
				// If there is none, continue
				point p(0.0, 0.0);
				if (!tools::is_intersecting(l1, l2, &p))
					continue;

				// If we're here, we have an intersection. The point where we got the intersection is
				// stored in p
				// Now, compute the length of the loop....
				double len(0.0);
				len += tools::dist_2p(p, path[n].coord);
				len += tools::dist_2p(path[m-1].coord, p);
				for (size_t i = n+1; i < m; ++i) {
					len += tools::dist_2p(path[i-1].coord, path[i].coord);
				}

				// If the length of the loop is below the threshold, then delete the loop by
				// replacing the coordinates of n with those of p and removing all nodes between n+1
				// and m-1.
				if (len <= max_len) {
					path[n].coord = p;

					for (size_t i = n+1; i < m; ++i) {
						path[i].to_erase = true;
					}
					path.reduce();
					++removed_intersections;

					// Restart inner loop at position n + 2.
					n = 1;
					m = n+2;
				}

			}
		}

		return removed_intersections;
	}


	// Return the simplified path
	pathType &get_simplified_path() {
		return simplified_path;
	}


private:

	// This is the paths we are operating on.
	pathType &input_path;
	pathType simplified_path;


};

}

#endif /* BYPASS_INTERSECTIONS_H_ */
