/******************************************************************************
* draw.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * draw.h
 *
 *  Created on: Jul 1, 2010
 *      Author: pajor
 */

#ifndef DRAW_H_
#define DRAW_H_

#include "../dataStructures/graph.h"

#include <boost/graph/graph_traits.hpp>
#include <boost/graph/iteration_macros.hpp>

#include <sstream>

// We draw through cairo :-)
#include <cairo/cairo.h>

namespace schematization {


template<typename graphType>
void draw_graph(cairo_t *cr, graphType &graph, const bool draw_street_names = false, const bool draw_marker = false) {

	// Define some types
	typedef typename graph_traits<graphType>::edge_iterator edge_iterator;
	typedef typename graph_traits<graphType>::vertex_iterator vertex_iterator;
	typedef typename graphType::vertex_descriptor vertex_descriptor;

	// If the graph has no edges, return :-)
	if (num_edges(graph) == 0)
		return;

	// Reset coord transformation on cr.
	cairo_identity_matrix(cr);

	// Define various drawing styles
	cairo_set_line_join(cr, CAIRO_LINE_JOIN_ROUND);
	cairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND);

	edge_iterator current_edge, last_edge;

	// Draw the borders
	col_black.set_source_rgba(cr);
	for (tie(current_edge, last_edge) = edges(graph); current_edge != last_edge; ++current_edge) {
		vertex_descriptor u(source(*current_edge, graph));
		vertex_descriptor v(target(*current_edge, graph));

		cairo_move_to(cr, graph[u].coord.x, graph[u].coord.y);
		cairo_set_line_width(cr, base_line_width*category_to_thickness[graph[*current_edge].category] + base_border_extent);
		cairo_line_to(cr, graph[v].coord.x, graph[v].coord.y);
		cairo_stroke(cr);
	}

	// Draw the actual edges
	for (tie(current_edge, last_edge) = edges(graph); current_edge != last_edge; ++current_edge) {
		vertex_descriptor u(source(*current_edge, graph));
		vertex_descriptor v(target(*current_edge, graph));	

		cairo_move_to(cr, graph[u].coord.x, graph[u].coord.y);
		category_to_color[graph[*current_edge].category].set_source_rgba(cr);
		cairo_set_line_width(cr, base_line_width*category_to_thickness[graph[*current_edge].category]);
		cairo_line_to(cr, graph[v].coord.x, graph[v].coord.y);
		cairo_stroke(cr);
	}

	// Draw the nodes
	vertex_iterator current_vertex, last_vertex;
	for (tie(current_vertex, last_vertex) = vertices(graph); current_vertex != last_vertex; ++current_vertex) {
		if (in_degree(*current_vertex, graph) + out_degree(*current_vertex, graph) == 1)
			continue;
		
		if (graph[*current_vertex].node_id == DUMMY_NODE_ID)
		    continue;	
		
		col_black.set_source_rgba(cr);
	
		cairo_arc(cr, graph[*current_vertex].coord.x, graph[*current_vertex].coord.y, base_node_thickness, 0, 2*pi);
		cairo_fill(cr);
	}

	
	
 }
	


}
#endif /* DRAW_H_ */
