/******************************************************************************
* tools.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * tools.h
 *
 *  Created on: 09.06.2010
 *      Author: pajor
 */

#ifndef TOOLS_H_
#define TOOLS_H_


#include "../types.h"
#include "point.h"
#include "line.h"
#include "latLng.h"

namespace schematization {

class tools {
public:

	// Convert degrees to radians
	template<typename T>
	static inline T deg_to_rad(const T &deg) {
		return deg * (pi / 180.0);
	}

	// Convert radians to degrees
	template<typename T>
	static inline T rad_to_deg(const T &rad) {
		return rad / (pi / 180.0);
	}


	// Square
	template<typename T>
	static inline T sqr(const T &x) {
		return x*x;
	}


	// Euclidean distance between two points
	static inline double dist_2p(const point &p1, const point &p2) {
		return sqrt(sqr(p1.x - p2.x) + sqr(p1.y - p2.y));
	}


	// Euclidean distance between three points
	static inline double dist_3p(const point &p1, const point &p2, const point &p3) {
		const double d = sqr(p2.x-p1.x) + sqr(p2.y-p1.y);
		const double u = ((p3.x-p1.x)*(p2.x-p1.x) + (p3.y-p1.y)*(p2.y-p1.y)) / d;
		const point p(p1.x + u*(p2.x-p1.x), p1.y + u*(p2.y-p1.y));

		// Compute distance between x3/y3 and x/y as the distance
		return tools::dist_2p(p3,p);
	}


	// Bound a value from above and below.
	template<typename T>
	static inline T bound(T val, T minimum, T maximum) {
		val = std::max(val, minimum);
		val = std::min(val, maximum);
		return val;
	}


	// Test for intersection between two lines.
	// If the lines intersect, the intersection is written to int_p
	static bool is_intersecting(const line l1, const line l2, point *int_p = NULL)  {
		// We say l1 = (u1, v1) and l2 = (u2, v2)
		// Thus we have [(xu1,yu1),(xv1,yv1)] and [(xu2,yu2),(xv2,yv2)]
		const double xu1 = l1.p1.x;
		const double yu1 = l1.p1.y;
		const double xv1 = l1.p2.x;
		const double yv1 = l1.p2.y;
		const double xu2 = l2.p1.x;
		const double yu2 = l2.p1.y;
		const double xv2 = l2.p2.x;
		const double yv2 = l2.p2.y;

		const double A1 = yv1 - yu1;
		const double B1 = xu1 - xv1;
		const double C1 = A1*xu1 + B1*yu1;
		const double A2 = yv2 - yu2;
		const double B2 = xu2 - xv2;
		const double C2 = A2*xu2 + B2*yu2;

		const double det = A1*B2 - A2*B1;

		bool intersect = true;

		// Lines are parallel, so no overlap
		if (det == 0)
			intersect = false;
		else {
			// If line Segments are not parallel then test for intersection...
			// Compute intersection point
			double x = (B2*C1 - B1*C2)/det;
			double y = (A1*C2 - A2*C1)/det;

			// Test whether intersection point is contained on the line segments
			double epsilon = 0.1; //circumenvents problems with float point arithmetic
			if (! (std::min(xu1,xv1) - epsilon <= x && x <= std::max(xu1,xv1) + epsilon) )
				intersect = false;
			if (! (std::min(yu1,yv1) - epsilon <= y && y <= std::max(yu1,yv1) + epsilon) )
				intersect = false;
			if (! (std::min(xu2,xv2) - epsilon <= x && x <= std::max(xu2,xv2) + epsilon) )
				intersect = false;
			if (! (std::min(yu2,yv2) - epsilon <= y && y <= std::max(yu2,yv2) + epsilon) )
				intersect = false;

			// If we have an intersection write coordinance of intersection point
			if (intersect) {
				if (int_p != NULL)
					int_p->x = x;
				if (int_p != NULL)
					int_p->y = y;
			}
		}

		// If we have an intersection, return true
		return intersect;
	}


	// Tests whether a bounding box is landscape or portrait oriented.
	static bool is_landscape(const boundingBox &bb) {
		return bb.get_extent_x() >= bb.get_extent_y();
	}

	// Retrieves the aspect ratio of a bounding box
	static double get_aspect_ratio(const boundingBox &bb) {
		return bb.get_extent_x() / bb.get_extent_y();
	}


	// Retrieves the quadrant (0..3) where the line lies in
	static int get_quadrant(const line &l) {
		const double phi = get_angle(l);

		int quadrant = int(std::floor( (phi) / deg_to_rad(90.0))); //GEMSA TODO: Vorläufig das const entfernt. Grund: s. unten
		   
		/*
		cout << l.p1.x << ", " << l.p1.y << " -- " << l.p2.x << ", " << l.p2.y << endl;
		cout << "winkel: " << phi * 180 / M_PI << endl;
        cout << "#quadrant: " << quadrant << endl;
        */
		
		//assert(quadrant >= 0 && quadrant <= 3);
		if (quadrant < 0 || quadrant > 3)
		{
			cout << "FIXME: Quadrant Error: " << phi << " " << quadrant << endl;
			//phi so groß, dass winkel > 360° rauskommt. Dann zerbricht alles.
			quadrant = quadrant % 4; //Notfallhack der zusichert, dass kein Fehler auftritt. Elegant ist aber anders.
		}
		return quadrant;
	}

	// Returns the angle of a line
	static double get_angle(const line &l) {
		// Compute angle
		assert(l.p2.x != l.p1.x || l.p1.y != l.p2.y); //should be avoided by "remove_superfluous_vertices.h" 
		double cos_phi = (l.p2.x - l.p1.x) / std::sqrt(sqr(l.p2.x-l.p1.x) + sqr(l.p2.y-l.p1.y));
		double phi = std::acos(cos_phi);

		// If we are below the x-axis, add 2pi to phi
		if (l.p2.y < l.p1.y)
			phi = 2.0 * pi - phi;

		return phi;
	}


	// Retrieve the preferred directions of a line given a smallest angle \alpha
	static int get_sector(const line &l, const angleType a) {
		const double a_rad = tools::deg_to_rad(a);
		const double phi = get_angle(l);
		const int sec = static_cast<int>(std::floor((phi + a_rad/2) / a_rad)) % static_cast<int>(360.0 / a);

		return sec;
	}
	
	static double round(double input, int nmb)
	{
		return (double) ( ( (int) (input * pow(10, nmb))) / pow(10, nmb) ) ;
	}

	static double distanceLatLong(const latLng& l1, const latLng& l2){
		const double earths_radius = 6371.0; //km
		const double lat1 = deg_to_rad(l1.lat);
		const double lat2 = deg_to_rad(l2.lat);
		/* const double long1 = deg_to_rad(l1.lng); */
		/* const double long2 = deg_to_rad(l2.lng); */
		const double dLat = deg_to_rad(l2.lat - l1.lat);
		const double dLon = deg_to_rad(l2.lng - l1.lng);

		double a = sin(dLat/2) * sin(dLat / 2) + cos(lat1) * cos(lat2) * sin(dLon/2) * sin(dLon/2);

		double c = 2 * atan2(sqrt(a), sqrt(1-a));
		return earths_radius * c; //returns distance in km
	}	
};

}

#endif /* TOOLS_H_ */
