/******************************************************************************
* graphSchematization.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * graphSchematization.h
 *
 *  Created on: Jul 5, 2010
 *      Author: pajor
 */

#ifndef GRAPHSCHEMATIZATION_H_
#define GRAPHSCHEMATIZATION_H_


#include "dataStructures/graph.h"
#include "time.h"
#include "algorithms/simplification.h"
#include "algorithms/draw.h"
#include "algorithms/scaling.h"
#include "axismonotone/schemePathWrapper.h"

#include "milp/schemMIP.h"

#include <gurobi_c++.h>
#include <cairo/cairo-pdf.h>
#include <boost/graph/graph_traits.hpp>
#include <boost/graph/iteration_macros.hpp>
namespace schematization {


  template<typename graphType>
    class graphSchematizer {

  public:

    // Initialize this class by passing an input graph
  graphSchematizer(graphType &g, int orthogonal_order_ = 1) :
    input_graph(g),
      simplified_graph(g),
      last_epsilon(0.0),
       last_angle(0),
       last_infeasible(false),
       last_running_time(0.0),
       last_num_iterations(0),
       orthogonal_order(orthogonal_order_) {

	 gurobi_env.set(GRB_IntParam_OutputFlag, 1); 	// Disable gurobi output
	 gurobi_env.set(GRB_IntParam_Threads, 1); //Only Single threaded execution
       }

     void schematizeWithAxisMonotoneAlgo(const angleType angle, const double eps) {
       simplify(eps);
       schemePathWrapper<graphType> spw;
       spw.schematize(simplified_graph, schematized_graph, angle, false);
     }

     void schematize(const angleType angle, const double eps) {
       simplify(eps);
       schematizeGraph(angle, eps);
    }

    /*
     * This draws a pdf image from the path
     */
    void writePDF(const string &filename) {
      cout << endl << "writing PDF file to " << filename << endl;
      // Initialize cairo for pdf
      cairo_surface_t *surface = cairo_pdf_surface_create(filename.c_str(), 1, 1);
      cairo_t *cr = cairo_create(surface);
      graphType G;
      cairo_pdf_surface_set_size(surface, paper_size_w, paper_size_h+paper_margin_h);
      cairo_show_page(cr);

      // draw input.
      G = input_graph;
      scale_graph_to_paper(G);
      cairo_pdf_surface_set_size(surface, paper_size_w, paper_size_h+paper_margin_h);
      draw_graph(cr, G, false);
      add_label(cr, paper_size_h, "Input Path");
      cairo_show_page(cr);
		
      // draw simplified graph
      if (num_vertices(simplified_graph) > 0){
	G = simplified_graph;
	scale_graph_to_paper(G);
	cairo_pdf_surface_set_size(surface, paper_size_w, paper_size_h+paper_margin_h);
	draw_graph(cr, G, false);
	add_label(cr, paper_size_h, "After applying Ramer-Douglas-Peucker Algorithm | Eps: " + boost::lexical_cast<string>(last_epsilon));
	cairo_show_page(cr);
      }
		
      // draw schematized graph.
      G = schematized_graph;
      scale_graph_to_paper(G); 
      cairo_pdf_surface_set_size(surface, paper_size_w, paper_size_h+paper_margin_h);
      draw_graph(cr, G, true, true);
      add_label(cr, paper_size_h, "Schematized | Edges: " +
		boost::lexical_cast<string>(num_edges(schematized_graph))
		+ " | Smallest Angle: " + boost::lexical_cast<string>(last_angle) + " deg"
		+ " | Running-Time: " + boost::lexical_cast<string>(last_running_time) + " ms"
		+ " | Infeasible: " + (last_infeasible ? "YES" : "NO")
		+ " | Iterations: " + boost::lexical_cast<string>(last_num_iterations));
      cairo_show_page(cr);
      // draw schematized graph.
		
      /*
	G = schematized_graph;
	boundingBox bb = get_graph_bounding_box(G);
	//scale_graph_to_paper(G); //vorher move_graph_to_paper
	move_graph_to_paper(G);
	//cairo_pdf_surface_set_size(surface, paper_size_w, paper_size_h+paper_margin_h);
	int max_w = std::max(paper_size_w, bb.right - bb.left + 5 * paper_margin_h);
	int max_h = std::max(paper_size_h + paper_margin_h, bb.bottom - bb.top + 5 * paper_margin_h);
		
	cairo_pdf_surface_set_size(surface, max_w , max_h);
	draw_graph(cr, G, true, true);
	add_label(cr, paper_size_h + 5, "Schematized | Edges: " +
	boost::lexical_cast<string>(num_edges(schematized_graph))
	+ " | Smallest Angle: " + boost::lexical_cast<string>(last_angle) + " deg"
	+ " | Running-Time: " + boost::lexical_cast<string>(last_running_time) + " ms"
	+ " | Infeasible: " + (last_infeasible ? "YES" : "NO")
	+ " | Iterations: " + boost::lexical_cast<string>(last_num_iterations));
	cairo_show_page(cr);
      */
      cairo_destroy(cr);
      cairo_surface_destroy(surface);
    }

  protected:

    // Adds labels to the path
    void add_label(cairo_t *cr, const double y_offset, const string &label) const {
      // Revert to identity matrix
      cairo_identity_matrix(cr);

      col_black.set_source_rgba(cr);

      // Move to bottom corner.
      cairo_move_to(cr, 2.0, y_offset + paper_margin_h - 2.0);
      cairo_set_font_size(cr, base_footline_font_size);
      cairo_show_text(cr, label.c_str());
    }

    // Simplifies the graph
    void simplify(const double eps) {
      Timer tau;
      cout << "Simplifying the graph... " << flush;
      simplify_graph(input_graph, simplified_graph, eps);
      cout << "done (" << tau.elapsed() << " ms)" << endl;
    }


    /*
     * This finally schematizes the graph.
     * Builds the model, optimizes the model and creates a schematized graph
     */
    void schematizeGraph(const angleType angle, const double eps) {
      if (num_vertices(simplified_graph) == 0) {
	cout << "input is an empty graph" << endl;
	return;
      }
      
      // Solve ILP on simplified graph
      Timer tau;
      cout << "Creating model class... " << flush;
      schemMIP<graphType> model(gurobi_env, simplified_graph, angle);
      cout << "done (" << tau.elapsed() << " ms)" << endl << endl;
      tau.restart();
      model.solve(schematized_graph);
      last_running_time   = tau.elapsed();
      last_infeasible     = model.is_solved();
      last_num_iterations = model.get_num_iterations();
      last_angle          = angle;
      last_epsilon        = eps;
    }


  private:

    // The gurobi environment
    GRBEnv gurobi_env;

    // This is the graphs we want to schematize
    graphType input_graph, simplified_graph, schematized_graph;

    // Some stuff
    double last_epsilon;
    angleType last_angle;
    bool last_infeasible;
    double last_running_time;
    unsigned int last_num_iterations;

    int orthogonal_order;
  };


}


#endif /* GRAPHSCHEMATIZATION_H_ */
