/******************************************************************************
* main.cpp
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
#include <iostream>
using namespace std;

#include "types.h"
#include "graphSchematization.h"
#include "schemStats.h"

#include "dataStructures/graph.h"
#include "algorithms/simplification.h"
#include "algorithms/draw.h"
#include "algorithms/scaling.h"
#include "geometry/point.h"

#include <gurobi_c++.h>
#include <boost/graph/graph_traits.hpp>
#include <boost/graph/iteration_macros.hpp>
#include <boost/program_options.hpp>

#include "milp/schemMIP.h"

#include <vector>


typedef schematization::schemGraph schemGraphType;


void schematizeFile(const string& in_filename, const string& out_filename, double angle, double eps, bool use_mip = true) {
  schemGraphType g;

  schematization::read_graph<schemGraphType>(g, in_filename);

  schematization::graphSchematizer< schemGraphType > schema(g);
  if (use_mip)
    schema.schematize(30, .5);
  else
    schema.schematizeWithAxisMonotoneAlgo(30, 0.5);
  
  schema.writePDF(out_filename);
}




void generateExamplePath(schemGraphType& out_graph){
  using namespace schematization;

  out_graph.clear();
  
  // Traverse path and add nodes (and edges) to the graph.
  typename schemGraphType::vertex_descriptor pre_node, new_node, pre_pre_node;
  typename schemGraphType::edge_descriptor new_edge;
  
  // Add first vertex to the output graph
  pre_node = add_vertex(out_graph);
  out_graph[pre_node].coord.x = 0;
  out_graph[pre_node].coord.y = 0;

  std::vector <point> coordinates{point(100, 100), point(200, 10), point(50, 50), point(750, -5)};

  int counter = 0;
  for (auto it = coordinates.begin(); it != coordinates.end(); ++it){
    //add new vertex
    new_node = add_vertex(out_graph);
    out_graph[new_node].coord.x = (*it).x;
    out_graph[new_node].coord.y = (*it).y;
    //out_graph[pre_node].node_id = counter;

    //add new edge
    bool edge_added = false;
    tie (new_edge, edge_added) = add_edge(pre_node, new_node, out_graph);
    out_graph[new_edge].edge_id = counter;
    out_graph[new_edge].name = "dummyname";


    //
    counter++;
    pre_node = new_node;
  }

  
}


int main(int ac, char* av[]) {

  std::cout << "Copyright (C) 2013; Andreas Gemsa, Thomas Pajor" << std::endl
	    << "This program comes with ABSOLUTELY NO WARRANTY; for details read the document located at http://www.gnu.org/licenses/gpl-3.0.html" << std::endl 
	    << "This is free software, and you are welcome to redistribute it under certain conditions; for details read the document located at http://www.gnu.org/licenses/gpl-3.0.html" << std::endl << std::endl;



  double angle, eps;
  string in_filename("");
  string out_pdf_filename("out.pdf");
  string directory("");
  bool use_heuristic(false);
  

  namespace po = boost::program_options;
  po::options_description desc("Allowed options");
  desc.add_options()
    ("help", "produce help message")
    ("angle,a", po::value<double>(&angle)->default_value(30.0), 
     "smallest angle (must divide 90° evenly")
    ("epsilon,e", po::value<double>(&eps)->default_value(0.125), 
     "epsilon parameter for douglas peucker [0.0 - 1.0]")
    ("heuristic,h", "use heuristic (default exact solution with mip)")
    ("infilename,i", po::value<string>(&in_filename)->implicit_value(""),
     "file to schematize")
    ("pdf,p", po::value<string>(&out_pdf_filename)->implicit_value(""),
     "output pdf filename")
    ("directory,d", po::value<string>(&directory)->implicit_value(""),
     "directory of files to schematize (only statistics)")
    ;

  po::variables_map vm;
  po::store(po::parse_command_line(ac, av, desc), vm);
  po::notify(vm);
    
  if (vm.count("help")) {
    cout << "Example: .\\main -a 30 -e 0.125 -i test.path -p test.pdf" << endl;
    cout << desc;
    return 0;
  }

  if (vm.count("heuristic") || vm.count("h")) {
      use_heuristic = true;
  }

  if (directory != "") {
    cout << "generate statistics for " << directory << endl;

    schematization::schemStats stats(angle, eps);
    stats.load_directory_nodefiles(directory);
    if (!use_heuristic) {
      stats.schematize();
      stats.dump_csv("stats.csv");
    }
    else {
      stats.schematizeAxsisMonotone();
      stats.dump_csv_axismonotone("stats.csv");
    }

    return 0;
  }

  if (in_filename != "") {
    schematizeFile(in_filename, out_pdf_filename, angle, eps, !use_heuristic);
    return 0;
  }

  cout << endl << "** no input specified! Use dummy example ** " << endl << endl;

  schemGraphType s;
  generateExamplePath(s);
  
  schematization::graphSchematizer< schemGraphType > g(s);
  if (!use_heuristic)
    g.schematize(angle, eps);
  else
    g.schematizeWithAxisMonotoneAlgo(angle, eps);

  g.writePDF(out_pdf_filename);
  return 0;
}

